#!/usr/bin/env node

'use strict';

var chalk = require('chalk');
var program = require('commander');
var fileSize = require('filesize');
var path = require('path');
var prettyMs = require('pretty-ms');
var __ = require('..');
var fs = require('fs');
var rollup = require('rollup');

var version = "0.8.1";

function createNext(arr, callback) {
  var index = 0;

  var next = function () {
    if (!(index in arr)) {
      return;
    }

    var currentIndex = index++;
    var item = arr[currentIndex];
    callback(item, next, currentIndex);
  };

  return next;
}

function oneByOne(arr, callback) {
  var next = createNext(arr, callback);
  next();
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function _continue(value, then) {
  return value && value.then ? value.then(then) : then(value);
}

function _async(f) {
  return function () {
    var arguments$1 = arguments;

    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments$1[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function build(configs, callbacks) {
  if (callbacks.start) {
    callbacks.start();
  }

  oneByOne(configs, _async(function (config, next, index) {
    var input = config.input;
    var outputOptions = config.output;
    var output = outputOptions.file;

    if (callbacks.buildStart) {
      callbacks.buildStart(input, output);
    }

    return _continue(_catch(function () {
      var currentTime = Date.now();
      return _await(rollup.rollup(config), function (buildResult) {
        return _await(buildResult.write(outputOptions), function () {
          var ref = fs.statSync(output);
          var size = ref.size;
          var totalTime = Date.now() - currentTime;

          if (callbacks.buildEnd) {
            callbacks.buildEnd(output, size, totalTime);
          }
        });
      });
    }, function (err) {
      if (callbacks.error) {
        callbacks.error(err);
      }
    }), function () {
      if (index + 1 >= configs.length) {
        if (callbacks.end) {
          callbacks.end();
        }
      }

      next();
    });
  }));
}

function watch(configs, callbacks) {
  var watcher = rollup.watch(configs);
  watcher.on("event", function (event) {
    var code = event.code;
    var input = event.input;
    var output = event.output;
    var duration = event.duration;
    var error = event.error;

    if (callbacks.start && code === "START") {
      callbacks.start();
    }

    if (callbacks.end && code === "END") {
      callbacks.end();
    }

    if (callbacks.buildStart && code === "BUNDLE_START") {
      for (var filename of output) {
        callbacks.buildStart(input, filename);
      }
    }

    if (callbacks.buildEnd && code === "BUNDLE_END") {
      for (var filename$1 of output) {
        var ref = fs.statSync(filename$1);
        var size = ref.size;
        callbacks.buildEnd(filename$1, size, duration || 0);
      }
    }

    if (callbacks.error && (code === "ERROR" || code === "FATAL")) {
      callbacks.error(error);
    }
  });
}

function _await$1(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var bundlib = _async$1(function (cwd, ref, callbacks) {
  var dev = ref.dev;
  var watch$1 = ref.watch;

  return _await$1(__.configsFromPkg(cwd, dev), function (_configsFromPkg) {
    (watch$1 ? watch : build)(_configsFromPkg, callbacks);
  });
});

function _async$1(f) {
  return function () {
    var arguments$1 = arguments;

    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments$1[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function log(msg) {
  console.log(msg);
}

function _empty() {}

function _awaitIgnored(value, direct) {
  if (!direct) {
    return value && value.then ? value.then(_empty) : Promise.resolve();
  }
}

function _catch$1(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function _continueIgnored(value) {
  if (value && value.then) {
    return value.then(_empty);
  }
}

function _async$2(f) {
  return function () {
    var arguments$1 = arguments;

    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments$1[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

function logError(err) {
  if (err.stack) {
    console.error(err.stack);
  }

  console.error(((chalk.bgRed.bold(" error ")) + " " + (err.message || err)));
}

var action = _async$2(function () {
  var cwd = process.cwd();
  var dev = program.dev;
  var watch = program.watch;
  var silent = program.silent;

  if (!silent) {
    log(((chalk.green.bold(("Bundlib v" + version))) + "\n"));
    log(((chalk.cyan.bold("reading package.json...")) + "\n"));
  }

  var buildIndex = 0;
  return _continueIgnored(_catch$1(function () {
    return _awaitIgnored(bundlib(cwd, {
      dev: dev,
      watch: watch
    }, Object.assign({}, {error: function error(err) {
        logError(err);
      }},

      (!silent && Object.assign({}, {buildEnd: function buildEnd(filename, size, duration) {
          log(((chalk.green.inverse.bold(" built ")) + " " + (chalk.yellow.bold(path.relative(cwd, filename))) + " ( " + (chalk.magenta.bold(fileSize(size))) + " " + (chalk.cyan("in")) + " " + (chalk.magenta.bold(prettyMs(duration, {
            secondsDecimalDigits: 2
          }))) + " )"));
        },

        error: function error(err) {
          logError(err);
        }},

        (watch && {
          start: function start() {
            if (buildIndex) {
              log(((chalk.cyan("rebuilding...")) + "\n"));
            }

            buildIndex++;
          },

          end: function end() {
            log(("\n" + (chalk.cyan("waiting for changes..."))));
          }

        }))))));
  }, function (err) {
    logError(err);
  }));
});

program.version(version, "-v, --version").option("-d, --dev", "create development builds").option("-w, --watch", "run bundlib in watch mode").option("-s, --silent", "prevent messages from showing in the console").action(action).parse(process.argv);
//# sourceMappingURL=bundlib-cli.js.map
